==> dismissible_message_bar-8.x-1.0.tar.gz: dismissible_message_bar/src/Service/DmbNotificationService.php <==
<?php

namespace Drupal\dismissible_message_bar\Service;

use DateTimeZone;
use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Render\Renderer;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\TypedData\Exception\MissingDataException;
use Drupal\Core\Url;
use Exception;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class DmbNotificationService.
 *
 * @package Drupal\dismissible_message_bar\Service
 */
class DmbNotificationService {

  use StringTranslationTrait;

  /**
   * Current path stack.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPathStack;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Renderer service.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * Language Manager interface.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The path matcher.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * An alias manager to find the alias for the current system path.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * Constructs a new NotificationsService object.
   *
   * @param \Drupal\Core\Path\CurrentPathStack $current_path_stack
   *   Current path stack.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Request Stack.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   Renderer service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language manager service.
   * @param \Drupal\Core\Path\PathMatcherInterface $path_matcher
   *   Path matcher service.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   Alias manager service.
   */
  public function __construct(
    CurrentPathStack $current_path_stack,
    EntityTypeManagerInterface $entity_type_manager,
    RequestStack $request_stack,
    Renderer $renderer,
    LanguageManagerInterface $language_manager,
    PathMatcherInterface $path_matcher,
    AliasManagerInterface $alias_manager) {

    $this->currentPathStack = $current_path_stack;
    $this->entityTypeManager = $entity_type_manager;
    $this->requestStack = $request_stack;
    $this->renderer = $renderer;
    $this->languageManager = $language_manager;
    $this->pathMatcher = $path_matcher;
    $this->aliasManager = $alias_manager;
  }

  /**
   * Return the render array with all relevant notifications.
   *
   * @return array
   *   Relevant notifications render array.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function returnRelevantNotifications() {
    $result = [];

    // Grab the notifications cookie.
    $request = $this->requestStack->getCurrentRequest();
    $params = $request->request->all();
    $notification_type = isset($params['notificationType']) ? $params['notificationType'] : FALSE;

    $notifications = $this->loadOpenNotifications($notification_type, $request);

    if (!empty($notifications)) {

      $this_content_type = isset($params['contentType']) ? $params['contentType'] : FALSE;

      /** @var \Drupal\dismissible_message_bar\Entity\DmbNotificationsEntityInterface $notification */
      foreach ($notifications as $notification) {

        // Check for sitewide notifications.
        $sitewide = '';
        if ($notification->hasField('field_sitewide') && !$notification->get('field_sitewide')->isEmpty()) {
          $sitewide = $notification->get('field_sitewide')->value;
        }

        // Check content type restrictions.
        if ($notification->hasField('field_content_types') && !$notification->get('field_content_types')->isEmpty()) {
          $allowed_content_types = [];
          foreach ($notification->get('field_content_types') as $content_type_ref) {
            $allowed_content_types[] = $content_type_ref->target_id;
          }

          // Skip this notification if the current page doesn't comply with
          // content type restrictions.
          if (empty($sitewide) && (!$this_content_type || ($this_content_type && !in_array($this_content_type, $allowed_content_types)))) {
            continue;
          }
          $is_on_allowed_content = TRUE;
        }

        // Validate page path restrictions.
        if ($notification->hasField('field_notification_pages') && !$notification->get('field_notification_pages')->isEmpty()) {
          $allowed_pages_raw = $notification->get('field_notification_pages')->value;
          $is_on_allowed_page = FALSE;

          $allowed_pages = explode(PHP_EOL, $allowed_pages_raw);

          foreach ($allowed_pages as $allowed_page) {
            // Convert path to lowercase. This allows comparison of the same
            // path with different case. Ex: /Page, /page, /PAGE.
            $pages = trim(mb_strtolower($allowed_page));

            if (!empty($pages)) {
              // Compare the lowercase path alias (if any) and internal path.
              $path = isset($params['contentPath']) ? $params['contentPath'] : '';
              // Do not trim a trailing slash if that is the complete path.
              $path = $path === '/' ? $path : rtrim($path, '/');

              // Check for NOT pages also
              $is_not_page = FALSE;
              if (substr($pages, 0, 1) === '!') {
                $is_not_page = TRUE;
                $pages = ltrim($pages, '!');
              }

              $path_alias = mb_strtolower($this->aliasManager->getAliasByPath($path));

              $on_mathing_pages = $this->pathMatcher->matchPath($path_alias, $pages) || (($path != $path_alias) && $this->pathMatcher->matchPath($path, $pages));
              if (!$is_not_page) {
                if ($on_mathing_pages) {
                  $is_on_allowed_page = TRUE;
                }
              } else {
                if ($on_mathing_pages) {
                  // This path is not allowed, disallow showing this notification.
                  $is_on_allowed_page = FALSE;
                  break;
                } else {
                  $is_on_allowed_page = TRUE;
                }
              }

            }
          }

          if (!$is_on_allowed_page) {
            continue;
          }

        }
        elseif (empty($sitewide) && empty($is_on_allowed_content)) {
          // This notification does not have a path set nor is it sitewide,
          // ignore it.
          continue;
        }

        // If the notification does not have a date range, set the start to be
        // 1 day ago and the end 1 day from now.
        $date_range = $notification->get('field_notification_date_range')->first();

        $gmtTimezone = new DateTimeZone('GMT');

        if (!empty($date_range->value)) {
          $start_date = new DrupalDateTime($date_range->value, $gmtTimezone);
          $date_range_begin = $start_date->getTimestamp();
        }
        else {
          $date_range_begin = strtotime('now - 1 day');
        }

        if (!empty($date_range->end_value)) {
          $end_date = new DrupalDateTime($date_range->end_value, $gmtTimezone);
          $date_range_end = $end_date->getTimestamp();
        }
        else {
          $date_range_end = strtotime('now + 1 day');
        }

        $now_time = new DrupalDateTime('now', $gmtTimezone);
        $now = $now_time->getTimestamp();

        // This notification has begun and has not ended.
        if ($now >= $date_range_begin && $now < $date_range_end) {
          // Build the view of referenced content.
          if (!isset($notification->field_p_content)) {
            // This notification does not have content, ignore it.
            continue;
          }

          $content = $notification->field_p_content;
          $viewBuilder = $this->entityTypeManager->getViewBuilder('paragraph');
          $content = $viewBuilder->viewField($content, ['label' => 'hidden']);

          $notification_id = $notification->id();

          // Automatically dismiss bar after set time
          if ($notification->hasField('field_auto_dismiss') && !$notification->get('field_auto_dismiss')->isEmpty()) {
            $auto_dismiss = $notification->get('field_auto_dismiss')->value;
            if ($notification->hasField('field_dismiss_time') && !$notification->get('field_dismiss_time')
                ->isEmpty()) {
              $dismiss_time = $notification->get('field_dismiss_time')->value;
            }
          }

          // Get the cookie expiration information.
          if ($notification->hasField('field_cookie_expiration') && !$notification->get('field_cookie_expiration')->isEmpty()) {
            $cookie_expiration = $notification->get('field_cookie_expiration')->value;
          }

          // Set the notification's classes.
          $classes = [
            'dmb-notification'
          ];
          // If notification has cookie turned off, set class.
          if ($notification->hasField('field_cookie_off') && !$notification->get('field_cookie_off')->isEmpty()) {
            $classes[] = 'cookie-off';
          }
          // Get the notification type (from term reference), set class.
          if ($notification->hasField('field_notification_type') && !$notification->get('field_notification_type')->isEmpty()) {
            $classes[] = strtolower(str_replace(' ', '-', $notification->get('field_notification_type')->entity->name->value));
          }

          $result[$notification_id] = [
            '#type' => 'container',
            '#attributes' => [
              'class' => $classes,
              'dmb-notification-id' => $notification_id,
              'dmb-cookie-expiration' => $cookie_expiration ?? 365,
              'dmb-auto-dismiss' => $auto_dismiss ?? FALSE,
              'dmb-dismiss-time' => $dismiss_time ?? 0,
            ],
            'notification' => [
              '#type' => 'container',
              '#attributes' => [
                'class' => ["dmb-notification-content"],
              ],
              'content' => $content,
            ],
            'close_notification' => [
              '#title' => $this->t('Close notification'),
              '#type' => 'link',
              '#url' => Url::fromUserInput('#'),
              '#attributes' => [
                'class' => ["close-dmb-notification"],
              ],
            ],
          ];
        }
      }
    }

    return $result;
  }

  /**
   * Load all notifications of the specified type.
   *
   * These are the ones that haven't been closed by the current user yet.
   *
   * @param string $type
   *   Notification type.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Current Request.
   *
   * @return array|\Drupal\Core\Entity\EntityInterface[]|int|static[]
   *   Loaded Notifications.
   */
  public function loadOpenNotifications($type, Request $request = NULL) {
    $notifications = [];
    $current_lang = $this->languageManager->getCurrentLanguage()->getId();

    // Grab the notifications cookie.
    if ($request) {

      // Check which notifications have already been closed by the user. They
      // should be excluded from being loaded later.
      $cookies = $request->cookies;
      if ($cookies && $cookies->get('dismissible_message_bar')) {
        $viewed_notifications = $cookies->get('dismissible_message_bar') ?: '';
        $viewed_notifications = explode('.', $viewed_notifications);
      }
    }

    try {
      $storage = $this->entityTypeManager->getStorage('dmb_notifications_entity');
      $query = $storage->getQuery()
        ->condition('type', 'default')
        ->condition('langcode', $current_lang)
        ->sort('created', 'DESC');

      if (!empty($viewed_notifications)) {
        $query->condition('id', $viewed_notifications, 'NOT IN');
      }

      if (!empty($type)) {
        // This notification block has a type requirement.
        $query->condition('field_notification_type', $type);
      }

      $notifications = $query->execute();
    }
    catch (InvalidPluginDefinitionException $e) {
      // Nothing to do, just prevent any edge case disruptions for now.
    }
    catch (PluginNotFoundException $e) {
      // Nothing to do, just prevent any edge case disruptions for now.
    }

    // Todo - this is quite heavy if there's ever a big notification amount.
    if (!empty($notifications) && isset($storage)) {
      $notifications = $storage->loadMultiple($notifications);

      /**
       * @var int $key
       * @var \Drupal\Core\Entity\EntityInterface $notification
       */
      foreach ($notifications as $key => $notification) {
        if ($notification->hasTranslation($current_lang)) {
          $notifications[$key] = $notification->getTranslation($current_lang);
        }
        else {
          unset($notifications[$key]);
        }
      }
    }

    return $notifications;
  }

  /**
   * Renders lazy loaded notifications from the notifications block.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Rendered Notifications.
   */
  public function renderNotifications() {

    $response_data = [];

    try {
      $notifications = $this->returnRelevantNotifications();
    }
    catch (MissingDataException $e) {
    }

    // Build notifications and add cacheable metadata.
    if (!empty($notifications)) {
      try {
        $notifications = $this->renderer->renderPlain($notifications);
        $response_data['notifications'] = $notifications;
      }
      catch (Exception $e) {
      }
    }

    $result = new JsonResponse($response_data);

    return $result;
  }

}

==> dismissible_message_bar-8.x-1.1.tar.gz: dismissible_message_bar/src/Service/DmbNotificationService.php <==
<?php

namespace Drupal\dismissible_message_bar\Service;

use DateTimeZone;
use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Render\Renderer;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\TypedData\Exception\MissingDataException;
use Drupal\Core\Url;
use Exception;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class DmbNotificationService.
 *
 * @package Drupal\dismissible_message_bar\Service
 */
class DmbNotificationService {

  use StringTranslationTrait;

  /**
   * Current path stack.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPathStack;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Renderer service.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * Language Manager interface.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The path matcher.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * An alias manager to find the alias for the current system path.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * Constructs a new NotificationsService object.
   *
   * @param \Drupal\Core\Path\CurrentPathStack $current_path_stack
   *   Current path stack.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Request Stack.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   Renderer service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language manager service.
   * @param \Drupal\Core\Path\PathMatcherInterface $path_matcher
   *   Path matcher service.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   Alias manager service.
   */
  public function __construct(
    CurrentPathStack $current_path_stack,
    EntityTypeManagerInterface $entity_type_manager,
    RequestStack $request_stack,
    Renderer $renderer,
    LanguageManagerInterface $language_manager,
    PathMatcherInterface $path_matcher,
    AliasManagerInterface $alias_manager) {

    $this->currentPathStack = $current_path_stack;
    $this->entityTypeManager = $entity_type_manager;
    $this->requestStack = $request_stack;
    $this->renderer = $renderer;
    $this->languageManager = $language_manager;
    $this->pathMatcher = $path_matcher;
    $this->aliasManager = $alias_manager;
  }

  /**
   * Return the render array with all relevant notifications.
   *
   * @return array
   *   Relevant notifications render array.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function returnRelevantNotifications() {
    $result = [];

    // Grab the notifications cookie.
    $request = $this->requestStack->getCurrentRequest();
    $params = $request->request->all();
    $notification_type = isset($params['notificationType']) ? $params['notificationType'] : FALSE;

    $notifications = $this->loadOpenNotifications($notification_type, $request);

    if (!empty($notifications)) {

      $this_content_type = isset($params['contentType']) ? $params['contentType'] : FALSE;

      /** @var \Drupal\dismissible_message_bar\Entity\DmbNotificationsEntityInterface $notification */
      foreach ($notifications as $notification) {

        // Check for sitewide notifications.
        $sitewide = '';
        if ($notification->hasField('field_sitewide') && !$notification->get('field_sitewide')->isEmpty()) {
          $sitewide = $notification->get('field_sitewide')->value;
        }

        // Check content type restrictions.
        if ($notification->hasField('field_content_types') && !$notification->get('field_content_types')->isEmpty()) {
          $allowed_content_types = [];
          foreach ($notification->get('field_content_types') as $content_type_ref) {
            $allowed_content_types[] = $content_type_ref->target_id;
          }

          // Skip this notification if the current page doesn't comply with
          // content type restrictions.
          if (empty($sitewide) && (!$this_content_type || !in_array($this_content_type, $allowed_content_types))) {
            continue;
          }
          $is_on_allowed_content = TRUE;
        }

        // Validate page path restrictions.
        if ($notification->hasField('field_notification_pages') && !$notification->get('field_notification_pages')->isEmpty()) {
          $allowed_pages_raw = $notification->get('field_notification_pages')->value;
          $is_on_allowed_page = FALSE;

          $allowed_pages = explode(PHP_EOL, $allowed_pages_raw);

          foreach ($allowed_pages as $allowed_page) {
            // Convert path to lowercase. This allows comparison of the same
            // path with different case. Ex: /Page, /page, /PAGE.
            $pages = trim(mb_strtolower($allowed_page));

            if (!empty($pages)) {
              // Compare the lowercase path alias (if any) and internal path.
              $path = isset($params['contentPath']) ? $params['contentPath'] : '/';
              // Do not trim a trailing slash if that is the complete path.
              $path = $path === '/' ? $path : rtrim($path, '/');
              // Check for NOT pages also.
              $is_not_page = FALSE;
              if (substr($pages, 0, 1) === '!') {
                $is_not_page = TRUE;
                $pages = ltrim($pages, '!');
              }

              $path_alias = mb_strtolower($this->aliasManager->getAliasByPath($path));

              $on_mathing_pages = $this->pathMatcher->matchPath($path_alias, $pages) || (($path != $path_alias) && $this->pathMatcher->matchPath($path, $pages));
              if (!$is_not_page) {
                if ($on_mathing_pages) {
                  $is_on_allowed_page = TRUE;
                }
              }
              else {
                if ($on_mathing_pages) {
                  // This path is not allowed, do not show this notification.
                  $is_on_allowed_page = FALSE;
                  break;
                }
                else {
                  $is_on_allowed_page = TRUE;
                }
              }

            }
          }

          if (!$is_on_allowed_page) {
            continue;
          }

        }
        elseif (empty($sitewide) && empty($is_on_allowed_content)) {
          // This notification does not have a path set nor is it sitewide,
          // ignore it.
          continue;
        }

        // If the notification does not have a date range, set the start to be
        // 1 day ago and the end 1 day from now.
        $date_range = $notification->get('field_notification_date_range')->first();

        $gmtTimezone = new DateTimeZone('GMT');

        if (!empty($date_range->value)) {
          $start_date = new DrupalDateTime($date_range->value, $gmtTimezone);
          $date_range_begin = $start_date->getTimestamp();
        }
        else {
          $date_range_begin = strtotime('now - 1 day');
        }

        if (!empty($date_range->end_value)) {
          $end_date = new DrupalDateTime($date_range->end_value, $gmtTimezone);
          $date_range_end = $end_date->getTimestamp();
        }
        else {
          $date_range_end = strtotime('now + 1 day');
        }

        $now_time = new DrupalDateTime('now', $gmtTimezone);
        $now = $now_time->getTimestamp();

        // This notification has begun and has not ended.
        if ($now >= $date_range_begin && $now < $date_range_end) {
          // Build the view of referenced content.
          if (!isset($notification->field_p_content)) {
            // This notification does not have content, ignore it.
            continue;
          }

          $content = $notification->field_p_content;
          $viewBuilder = $this->entityTypeManager->getViewBuilder('paragraph');
          $content = $viewBuilder->viewField($content, ['label' => 'hidden']);

          $notification_id = $notification->id();

          // Automatically dismiss bar after set time.
          if ($notification->hasField('field_auto_dismiss') && !$notification->get('field_auto_dismiss')->isEmpty()) {
            $auto_dismiss = $notification->get('field_auto_dismiss')->value;
            if ($notification->hasField('field_dismiss_time') && !$notification->get('field_dismiss_time')->isEmpty()) {
              $dismiss_time = $notification->get('field_dismiss_time')->value;
            }
          }

          // Get the cookie expiration information.
          if ($notification->hasField('field_cookie_expiration') && !$notification->get('field_cookie_expiration')->isEmpty()) {
            $cookie_expiration = $notification->get('field_cookie_expiration')->value;
          }

          // Set the notification's classes.
          $classes = [
            'dmb-notification',
          ];
          // If notification has cookie turned off, set class.
          if ($notification->hasField('field_cookie_off') && !$notification->get('field_cookie_off')->isEmpty()) {
            $classes[] = 'cookie-off';
          }
          // Get the notification type (from term reference), set class.
          if ($notification->hasField('field_notification_type') && !$notification->get('field_notification_type')->isEmpty()) {
            $classes[] = strtolower(str_replace(' ', '-', $notification->get('field_notification_type')->entity->name->value));
          }

          $result[$notification_id] = [
            '#type' => 'container',
            '#attributes' => [
              'class' => $classes,
              'dmb-notification-id' => $notification_id,
              'dmb-cookie-expiration' => $cookie_expiration ?? 365,
              'dmb-auto-dismiss' => $auto_dismiss ?? FALSE,
              'dmb-dismiss-time' => $dismiss_time ?? 0,
            ],
            'notification' => [
              '#type' => 'container',
              '#attributes' => [
                'class' => ["dmb-notification-content"],
              ],
              'content' => $content,
            ],
            'close_notification' => [
              '#title' => $this->t('Close notification'),
              '#type' => 'link',
              '#url' => Url::fromUserInput('#'),
              '#attributes' => [
                'class' => ["close-dmb-notification"],
              ],
            ],
          ];
        }
      }
    }

    return $result;
  }

  /**
   * Load all notifications of the specified type.
   *
   * These are the ones that haven't been closed by the current user yet.
   *
   * @param string $type
   *   Notification type.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Current Request.
   *
   * @return array|\Drupal\Core\Entity\EntityInterface[]|int|static[]
   *   Loaded Notifications.
   */
  public function loadOpenNotifications($type, Request $request = NULL) {
    $notifications = [];
    $current_lang = $this->languageManager->getCurrentLanguage()->getId();

    // Grab the notifications cookie.
    if ($request) {

      // Check which notifications have already been closed by the user. They
      // should be excluded from being loaded later.
      $cookies = $request->cookies;
      if ($cookies && $cookies->get('dismissible_message_bar')) {
        $viewed_notifications = $cookies->get('dismissible_message_bar') ?: '';
        $viewed_notifications = explode('.', $viewed_notifications);
      }
    }

    try {
      $storage = $this->entityTypeManager->getStorage('dmb_notifications_entity');
      $query = $storage->getQuery()
        ->condition('type', 'default')
        ->condition('langcode', $current_lang)
        ->sort('created', 'DESC');

      if (!empty($viewed_notifications)) {
        $query->condition('id', $viewed_notifications, 'NOT IN');
      }

      if (!empty($type)) {
        // This notification block has a type requirement.
        $query->condition('field_notification_type', $type);
      }

      $notifications = $query->execute();
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
      // Nothing to do, just prevent any edge case disruptions for now.
    }

    // Todo - this is quite heavy if there's ever a big notification amount.
    if (!empty($notifications) && isset($storage)) {
      $notifications = $storage->loadMultiple($notifications);

      /**
       * @var int $key
       * @var \Drupal\Core\Entity\EntityInterface $notification
       */
      foreach ($notifications as $key => $notification) {
        if ($notification->hasTranslation($current_lang)) {
          $notifications[$key] = $notification->getTranslation($current_lang);
        }
        else {
          unset($notifications[$key]);
        }
      }
    }

    return $notifications;
  }

  /**
   * Renders lazy loaded notifications from the notifications block.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Rendered Notifications.
   */
  public function renderNotifications() {

    $response_data = [];

    try {
      $notifications = $this->returnRelevantNotifications();
    }
    catch (MissingDataException $e) {
    }

    // Build notifications and add cacheable metadata.
    if (!empty($notifications)) {
      try {
        $notifications = $this->renderer->renderPlain($notifications);
        $response_data['notifications'] = $notifications;
      }
      catch (Exception $e) {
      }
    }

    $result = new JsonResponse($response_data);

    return $result;
  }

}
